use super::super::types::{HeaderEncoder, set_member_value};
use tool_helper::{bits::BitRange, raw::RawConversion, Error};

#[repr(packed(1))]
#[allow(dead_code)]
pub struct Header {
    value: u32
}

#[allow(dead_code)]
impl Header {
    const TEX_WIDTH_BIT_RANGE:   BitRange = BitRange::from_to(0, 8);
    const TEX_HEIGHT_BIT_RANGE:  BitRange = BitRange::from_to(9, 16);
    const CLUT_WIDTH_BIT_RANGE:  BitRange = BitRange::from_to(17, 22);
    const CLUT_HEIGHT_BIT_RANGE: BitRange = BitRange::from_to(23, 31);

    pub fn from(tex_width: u16, tex_height: u16, clut_width: u16, clut_height: u16) -> Result<Header, Error> {
        if tex_width & 1 == 1 || tex_height & 1 == 1 {
            Err(Error::from_text(format!("Image size (width: {}, height: {}) needs to be even", tex_width, tex_height)))
        }

        else {
            let value = set_member_value!(set_member_value!(set_member_value!(set_member_value!(0, 
                tex_width,   1, u32),
                tex_height,  1, u32),
                clut_width,  4, u32),
                clut_height, 0, u32);

            Ok(Header{value})
        }
    }
}

impl Default for Header {
    fn default() -> Self {
        Header{value: 0}
    }
}

impl HeaderEncoder for Header {
    fn encode(&self, tex_width: u16, tex_height: u16, clut_width: u16, clut_height: u16) -> Result<std::vec::Vec<u8>, Error> {
        Ok(Header::from(tex_width, tex_height, clut_width, clut_height)?.convert_to_raw().to_vec())
    }
}

impl RawConversion<4> for Header {
    fn convert_to_raw(&self) -> [u8; 4] {
        self.value.to_le_bytes()
    }
}